import { UserId } from '../../user/types';
import { z } from 'zod';
import { userBooksQueryParser } from './parsers';
import { BookType } from '../types';

export type UserBooksQuery = z.infer<typeof userBooksQueryParser>;

interface UserBooksReportRow {
  userId: UserId;
  books: {
    [t in BookType]: {
      // TODO debit/credit
      sumFrom: bigint;
      sumTo: bigint;
      // TODO proper type
    };
  };
}

export interface UserBooksReportResponse {
  rows: readonly UserBooksReportRow[];
  totals: UserBooksReportRow['books'];
  count: number;
}

type BookToJson<T extends { sumTo: bigint; sumFrom: bigint }> = Omit<T, 'sumTo' | 'sumFrom'> & {
  sumTo: string;
  sumFrom: string;
};

type BooksToJson<
  T extends {
    [k in BookType]: B;
  },
  B extends { sumTo: bigint; sumFrom: bigint } = T[BookType]
> = {
  [k in BookType]: BookToJson<T[k]>;
};

// bigints to json, TODO a little bit problematic
export interface UserBooksReportResponseSerialized {
  rows: (Omit<UserBooksReportResponse['rows'][number], 'books'> & {
    books: BooksToJson<UserBooksReportResponse['rows'][number]['books']>;
  })[];
  totals: BooksToJson<UserBooksReportResponse['totals']>;
  count: UserBooksReportResponse['count'];
}
